/**
 * AsaSDK.h
 * Include file of Asagiri SDK.
 * Copyright 2004 Comona Co., Ltd. <http://www.comona.co.jp/>
 * DLL file : Asagiri.dll
 * Import library : Asagiri.lib
 * Written by Toru Kageyama<kageyama@comona.co.jp>
 */

#ifndef __INCLUDE_ASAGIRI_SDK__
#define __INCLUDE_ASAGIRI_SDK__

/**
 * ARGS for nType member of StartEnumAsagiri() function.
 */
#define ASAITEMTYPE_POPUP				0x0
#define ASAITEMTYPE_STRING				0x1
#define ASAITEMTYPE_STRINGFILE			0x2
#define ASAITEMTYPE_STRINGALL			(ASAITEMTYPE_STRING | ASAITEMTYPE_STRINGFILE)
#define ASAITEMTYPE_BITMAP				0x4
#define ASAITEMTYPE_BITMAPFILE			0x8
#define ASAITEMTYPE_BITMAPALL			(ASAITEMTYPE_BITMAP | ASAITEMTYPE_BITMAPFILE)
#define ASAITEMTYPE_BINARY				0x10
#define ASAITEMTYPE_BINARYFILE			0x20
#define ASAITEMTYPE_BINARYALL			(ASAITEMTYPE_BINARY | ASAITEMTYPE_BINARYFILE)
#define ASAITEMTYPE_ALL					(ASAITEMTYPE_STRINGALL | ASAITEMTYPE_BITMAPALL | ASAITEMTYPE_BINARYALL)

/**
 * Return values for lpnType member of AsagiriEnumNext() function.
 */
#define ASATYPE_UNKNOWN			0
#define ASATYPE_POPUP			1
#define ASATYPE_STRING			2
#define ASATYPE_STRINGFILE		3
#define ASATYPE_BITMAP			4
#define ASATYPE_BITMAPFILE		5
#define ASATYPE_BINARY			6
#define ASATYPE_BINARYFILE		7

/**
 * Function protoypes.
 */
#ifdef __cplusplus
extern "C" {
#endif

	/**
	 * @1
	 * Initializes Asagiri SDK DLL.
	 * returns: Handle to Asagiri DLL data.
	 * args: none.
	 */
	HANDLE WINAPI AsagiriInitialize();

	/**
	 * @2
	 * Uninitializes Asagiri SDK DLL.
	 * returns: none.
	 * args: HANDLE hAsagiri; Handle to Asagiri DLL data.
	 */
	void WINAPI AsagiriUninitialize(HANDLE hAsagiri);

	/**
	 * @3
	 * Start enumlation of Asagiri items.
	 * returns: Handle to enumlate child items.
	 * args: HANDLE hAsagiri; Handle to Asagiri DLL data.
	 *		 short nPrentID; ID of parent item, the root item is 0. 
	 *		 short nType; Required type of items.  See ASAITEMTYPE_XXX above.
	 */
	HANDLE WINAPI AsagiriStartEnum(HANDLE hAsagiri, short nParentID, short nType);

	/**
	 * @4
	 * Forward position of item enumlation. (ASCII or DBCS)
	 * returns: ID of Asagiri item.
	 * args: HANDLE hAsagiri; Handle to Asagiri DLL data.
	 *		 HANDLE hEnum; Handle to enumlate Asagiri items returned by StartEnumAsagiri() function.
	 *		 LPSTR* lplpszTitle; Receives title string of current Asagiri item.
	 *		 short* lpnType; Receives type identifier of Asagiri item which is one of ASAITEMTYPE_XXX.
	 *		 short* lpnTemp; Receives life cycle of Asagiri item non-zero is temporary zero is stored into file.
	 */
	short WINAPI AsagiriEnumNextA(HANDLE hAsagiri, HANDLE hEnum, LPSTR* lplpszTitle, short* lpnType, short* lpnTemp);

	/**
	 * @5
	 * Forward position of item enumlation. (UNICODE)
	 * returns: ID of Asagiri item.
	 * args: HANDLE hAsagiri; Handle to Asagiri DLL data.
	 *		 HANDLE hEnum; Handle to enumlate Asagiri items returned by StartEnumAsagiri() function.
	 *		 LPWSTR* lplpszTitle; Receives title string of current Asagiri item.
	 *		 short* lpnType; Receives type identifier of Asagiri item which is one of ASAITEMTYPE_XXX.
	 *		 short* lpnTemp; Receives life cycle of Asagiri item non-zero is temporary zero is stored into file.
	 */
	short WINAPI AsagiriEnumNextW(HANDLE hAsagiri, HANDLE hEnum, LPWSTR* lplpszTitle, short* lpnType, short* lpnTemp);

	/**
	 * @6
	 * Reset enumlation items to the top.
	 * returns: Non-zero when succeed, zero when failed.
	 * args: HANDLE hAsagiri; Handle to Asagiri DLL data.
	 *		 HANDLE hEnum; Handle to enumlate Asagiri items returned by StartEnumAsagiri() function.
	 */
	BOOL WINAPI AsagiriResetEnum(HANDLE hAsagiri, HANDLE hEnum);

	/**
	 * @7
	 * End enumlation of Asagiri items.
	 * returns: Handle to enumlate child items.
	 * args: HANDLE hAsagiri; Handle to Asagiri DLL data.
	 *		 HANDLE hEnum; Handle to enumlate Asagiri items returned by StartEnumAsagiri() function.
	 */
	BOOL WINAPI AsagiriEndEnum(HANDLE hAsagiri, HANDLE hEnum);

	/**
	 * @8
	 * Get text value from Asagiri item. Valid for ASAITEMTYPE_STRING and ASAITEMTYPE_STRINGFILE items. (ASCII or DBCS)
	 * returns: Non-zero when succeed, zero when failed.
	 * args: HANDLE hAsagiri; Handle to Asagiri DLL data.
	 *		 short nID; ID of Asagiri item which string data will retreived.
	 *		 LPSTR* lplpszValue; Receives value string of Asagiri item.
	 *		 LPSTR* lplpszPathName; Receives file name string of Asagiri item.
	 */
	BOOL WINAPI AsagiriGetTextA(HANDLE hAsagiri, short nID, LPSTR* lplpszValue, LPSTR* lplpszPathName);

	/**
	 * @9
	 * Get text value from Asagiri item. Valid for ASAITEMTYPE_STRING and ASAITEMTYPE_STRINGFILE items. (UNICODE)
	 * returns: Non-zero when succeed, zero when failed.
	 * args: HANDLE hAsagiri; Handle to Asagiri DLL data.
	 *		 short nID; ID of Asagiri item which string data will retreived.
	 *		 LPWSTR* lplpszValue; Receives value string of Asagiri item.
	 *		 LPWSTR* lplpszPathName; Receives file name string of Asagiri item.
	 */
	BOOL WINAPI AsagiriGetTextW(HANDLE hAsagiri, short nID, LPWSTR* lplpszValue, LPWSTR* lplpszPathName);

	/**
	 * @10
	 * Get bitmap from Asagiri item. Valid for ASAITEMTYPE_BITMAP and ASAITEMTYPE_BITMAPFILE items. (ASCII or DBCS)
	 * returns: Non-zero when succeed, zero when failed.
	 * args: HANDLE hAsagiri; Handle to Asagiri DLL data.
	 *		 short nID; ID of Asagiri item which bitmap data will retreived.
	 *		 HBITMAP* phBitmap; Receives value HBTIMAP of Asagiri item.
	 *		 LPSTR* lplpszPathName; Receives file name string of Asagiri item.
	 */
	BOOL WINAPI AsagiriGetBitmapA(HANDLE hAsagiri, short nID, HBITMAP* phBitmap, LPSTR* lplpszPathName);

	/**
	 * @11
	 * Get bitmap from Asagiri item. Valid for ASAITEMTYPE_BITMAP and ASAITEMTYPE_BITMAPFILE items. (UNICODE)
	 * returns: Non-zero when succeed, zero when failed.
	 * args: HANDLE hAsagiri; Handle to Asagiri DLL data.
	 *		 short nID; ID of Asagiri item which bitmap data will retreived.
	 *		 HBITMAP* phBitmap; Receives value HBTIMAP of Asagiri item.
	 *		 LPWSTR* lplpszPathName; Receives file name string of Asagiri item.
	 */
	BOOL WINAPI AsagiriGetBitmapW(HANDLE hAsagiri, short nID, HBITMAP* phBitmap, LPWSTR* lplpszPathName);

	/**
	 * @12
	 * Get binary stream from Asagiri item. Valid for ASAITEMTYPE_BINARY and ASAITEMTYPE_BINARYFILE items. (ASCII or DBCS)
	 * returns: Non-zero when succeed, zero when failed.
	 * args: HANDLE hAsagiri; Handle to Asagiri DLL data.
	 *		 short nID; ID of Asagiri item which string binary will retreived.
	 *		 LPBYTE* lplpBytes; Receives value binary stream of Asagiri item.
	 *		 LPDWORD lpdwSize; Receives size of lplpBytes in byte.
	 *		 LPSTR* lplpszPathName; Receives file name string of Asagiri item.
	 */
	BOOL WINAPI AsagiriGetBinaryA(HANDLE hAsagiri, short nID, LPBYTE* lplpBytes, LPDWORD lpdwSize, LPSTR* lplpszPathName);

	/**
	 * @13
	 * Get binary stream from Asagiri item. Valid for ASAITEMTYPE_BINARY and ASAITEMTYPE_BINARYFILE items. (UNICODE)
	 * returns: Non-zero when succeed, zero when failed.
	 * args: HANDLE hAsagiri; Handle to Asagiri DLL data.
	 *		 short nID; ID of Asagiri item which string binary will retreived.
	 *		 LPBYTE* lplpBytes; Receives value binary stream of Asagiri item.
	 *		 LPDWORD lpdwSize; Receives size of lplpBytes in byte.
	 *		 LPWSTR* lplpszPathName; Receives file name string of Asagiri item.
	 */
	BOOL WINAPI AsagiriGetBinaryW(HANDLE hAsagiri, short nID, LPBYTE* lplpBytes, LPDWORD lpdwSize, LPWSTR* lplpszPathName);

	/**
	 * @14
	 * Get binary stream of image files from Asagiri item. Valid for ASAITEMTYPE_BITMAPFILE items. (ASCII or DBCS)
	 * returns: Non-zero when succeed, zero when failed.
	 * args: HANDLE hAsagiri; Handle to Asagiri DLL data.
	 *		 short nID; ID of Asagiri item which binary data will retreived.
	 *		 LPBYTE* lplpBytes; Receives value binary stream of Asagiri item.
	 *		 LPDWORD lpdwSize; Receives size of lplpBytes in byte.
	 *		 LPSTR* lplpszPathName; Receives file name string of Asagiri item.
	 */
	BOOL WINAPI AsagiriGetImageFileInBinaryA(HANDLE hAsagiri, short nID, LPBYTE* lplpBytes, LPDWORD lpdwSize, LPSTR* lplpszPathName);

	/**
	 * @15
	 * Get binary stream of image files from Asagiri item. Valid for ASAITEMTYPE_BITMAPFILE items. (UNICODE)
	 * returns: Non-zero when succeed, zero when failed.
	 * args: HANDLE hAsagiri; Handle to Asagiri DLL data.
	 *		 short nID; ID of Asagiri item which binary data will retreived.
	 *		 LPBYTE* lplpBytes; Receives value binary stream of Asagiri item.
	 *		 LPDWORD lpdwSize; Receives size of lplpBytes in byte.
	 *		 LPWSTR* lplpszPathName; Receives file name string of Asagiri item.
	 */
	BOOL WINAPI AsagiriGetImageFileInBinaryW(HANDLE hAsagiri, short nID, LPBYTE* lplpBytes, LPDWORD lpdwSize, LPWSTR* lplpszPathName);

	/**
	 * @16
	 * Get file name. (ASCII or DBCS)
	 * returns: Non-zero when succeed, zero when failed.
	 * args: HANDLE hAsagiri; Handle to Asagiri DLL data.
	 *		 short nID; ID of Asagiri item which binary data will retreived.
	 *		 LPSTR* lplpszPathName; Receives file name string of Asagiri item.
	 */
	BOOL WINAPI AsagiriGetFileNameA(HANDLE hAsagiri, short nID, LPSTR* lplpszPathName);

	/**
	 * @17
	 * Get file name. (UNICODE)
	 * returns: Non-zero when succeed, zero when failed.
	 * args: HANDLE hAsagiri; Handle to Asagiri DLL data.
	 *		 short nID; ID of Asagiri item which binary data will retreived.
	 *		 LPWSTR* lplpszPathName; Receives file name string of Asagiri item.
	 */
	BOOL WINAPI AsagiriGetFileNameW(HANDLE hAsagiri, short nID, LPWSTR* lplpszPathName);

	/**
	 * @18
	 * Add popup item. (ASCII or DBCS)
	 * returns: ID for added item if succeed, less than zero if failed.
	 * args: HANDLE hAsagiri; Handle to Asagiri DLL data.
	 *		 short nParentID; ID of Asagiri item which child item is about to add. This must be ASAITEMTYPE_POPUP item.
	 *		 LPCSTR lpszTitle; Name of Asagiri item.
	 */
	short WINAPI AsagiriAddPopupA(HANDLE hAsagiri, short nParentID, LPCSTR lpszTitle);

	/**
	 * @19
	 * Add popup item. (UNICODE)
	 * returns: ID for added item if succeed, less than zero if failed.
	 * args: HANDLE hAsagiri; Handle to Asagiri DLL data.
	 *		 short nParentID; ID of Asagiri item which child item is about to add. This must be ASAITEMTYPE_POPUP item.
	 *		 LPCWSTR lpszTitle; Name of Asagiri item.
	 */
	short WINAPI AsagiriAddPopupW(HANDLE hAsagiri, short nParentID, LPCWSTR lpszTitle);

	/**
	 * @20
	 * Add text item. (ASCII or DBCS)
	 * returns: ID for added item if succeed, less than zero if failed.
	 * args: HANDLE hAsagiri; Handle to Asagiri DLL data.
	 *		 short nParentID; ID of Asagiri item which child item is about to add. This must be ASAITEMTYPE_POPUP item.
	 *		 LPCSTR lpszTitle; Name of Asagiri item.
	 *		 LPCSTR lpszText; Value of Asagiri item.
	 */
	short WINAPI AsagiriAddTextA(HANDLE hAsagiri, short nParentID, LPCSTR lpszTitle, LPCSTR lpszText);

	/**
	 * @21
	 * Add text item. (UNICODE)
	 * returns: ID for added item if succeed, less than zero if failed.
	 * args: HANDLE hAsagiri; Handle to Asagiri DLL data.
	 *		 short nParentID; ID of Asagiri item which child item is about to add. This must be ASAITEMTYPE_POPUP item.
	 *		 LPCWSTR lpszTitle; Name of Asagiri item.
	 *		 LPCWSTR lpszText; Value of Asagiri item.
	 */
	short WINAPI AsagiriAddTextW(HANDLE hAsagiri, short nParentID, LPCWSTR lpszTitle, LPCWSTR lpszText);

	/**
	 * @22
	 * Add bitmap item. (ASCII or DBCS)
	 * returns: ID for added item if succeed, less than zero if failed.
	 * args: HANDLE hAsagiri; Handle to Asagiri DLL data.
	 *		 short nParentID; ID of Asagiri item which child item is about to add. This must be ASAITEMTYPE_POPUP item.
	 *		 LPCSTR lpszTitle; Name of Asagiri item.
	 *		 HBITMAP hBitmap; Value of Asagiri item.
	 */
	short WINAPI AsagiriAddBitmapA(HANDLE hAsagiri, short nParentID, LPCSTR lpszTitle, HBITMAP hBitmap);

	/**
	 * @23
	 * Add bitmap item. (UNICODE)
	 * returns: ID for added item if succeed, less than zero if failed.
	 * args: HANDLE hAsagiri; Handle to Asagiri DLL data.
	 *		 short nParentID; ID of Asagiri item which child item is about to add. This must be ASAITEMTYPE_POPUP item.
	 *		 LPCSTR lpszTitle; Name of Asagiri item.
	 *		 HBITMAP hBitmap; Value of Asagiri item.
	 */
	short WINAPI AsagiriAddBitmapW(HANDLE hAsagiri, short nParentID, LPCWSTR lpszTitle, HBITMAP hBitmap);

	/**
	 * @24
	 * Add binary item. (ASCII or DBCS)
	 * returns: ID for added item if succeed, less than zero if failed.
	 * args: HANDLE hAsagiri; Handle to Asagiri DLL data.
	 *		 short nParentID; ID of Asagiri item which child item is about to add. This must be ASAITEMTYPE_POPUP item.
	 *		 LPCSTR lpszTitle; Name of Asagiri item.
	 *		 LPCBYTE lpByte; Value of Asagiri item.
	 *		 DWORD dwSize; Size of lpBytem.
	 */
	short WINAPI AsagiriAddBinaryA(HANDLE hAsagiri, short nParentID, LPCSTR lpszTitle, const BYTE* lpByte, DWORD dwSize);

	/**
	 * @25
	 * Add binary item. (UNICODE)
	 * returns: ID for added item if succeed, less than zero if failed.
	 * args: HANDLE hAsagiri; Handle to Asagiri DLL data.
	 *		 short nParentID; ID of Asagiri item which child item is about to add. This must be ASAITEMTYPE_POPUP item.
	 *		 LPCWSTR lpszTitle; Name of Asagiri item.
	 *		 LPCBYTE lpByte; Value of Asagiri item.
	 *		 DWORD dwSize; Size of lpBytem.
	 */
	short WINAPI AsagiriAddBinaryW(HANDLE hAsagiri, short nParentID, LPCWSTR lpszTitle, const BYTE* lpByte, DWORD dwSize);

	/**
	 * @26
	 * Add binary item with file name assigned. (ASCII or DBCS)
	 * returns: ID for added item if succeed, less than zero if failed.
	 * args: HANDLE hAsagiri; Handle to Asagiri DLL data.
	 *		 short nParentID; ID of Asagiri item which child item is about to add. This must be ASAITEMTYPE_POPUP item.
	 *		 LPCSTR lpszTitle; Name of Asagiri item.
	 *		 LPCSTR lpszPathName; File path name. This must be full path.
	 */
	short WINAPI AsagiriAddFileA(HANDLE hAsagiri, short nParentID, LPCSTR lpszTitle, LPCSTR lpszPathName);

	/**
	 * @27
	 * Add binary item with file name assigned. (UNICODE)
	 * returns: ID for added item if succeed, less than zero if failed.
	 * args: HANDLE hAsagiri; Handle to Asagiri DLL data.
	 *		 short nParentID; ID of Asagiri item which child item is about to add. This must be ASAITEMTYPE_POPUP item.
	 *		 LPCWSTR lpszTitle; Name of Asagiri item.
	 *		 LPCWSTR lpszPathName; File path name. This must be full path.
	 */
	short WINAPI AsagiriAddFileW(HANDLE hAsagiri, short nParentID, LPCWSTR lpszTitle, LPCWSTR lpszPathName);

	/**
	 * @28
	 * Remove item.
	 * returns: ID for added item if succeed, less than zero if failed.
	 * args: HANDLE hAsagiri; Handle to Asagiri DLL data.
	 *		 short nID; ID of Asagiri item which is about to remove.
	 */
	BOOL WINAPI AsagiriRemoveItem(HANDLE hAsagiri, short nID);

	/**
	 * @29
	 * Delete ASCII or DBCS string.
	 * returns: none.
	 * args: HANDLE hAsagiri; Handle to Asagiri DLL data.
	 *		 LPSTR lpszString; String to delete.
	 */
	HRESULT WINAPI AsagiriGetLastComResult(HANDLE hAsagiri);

	/**
	 * @30
	 * Delete ASCII or DBCS string.
	 * returns: none.
	 * args: HANDLE hAsagiri; Handle to Asagiri DLL data.
	 *		 LPSTR lpszString; String to delete.
	 */
	void WINAPI AsagiriDeleteStringA(HANDLE hAsagiri, LPSTR lpszString);

	/**
	 * @31
	 * Delete UNICODE string.
	 * returns: none.
	 * args: HANDLE hAsagiri; Handle to Asagiri DLL data.
	 *		 LPWSTR lpszString; String to delete.
	 */
	void WINAPI AsagiriDeleteStringW(HANDLE hAsagiri, LPWSTR lpszString);

	/**
	 * @32
	 * Delete binary stream.
	 * returns: none.
	 * args: HANDLE hAsagiri; Handle to Asagiri DLL data.
	 *		 LPBYTE lpBin; Binary stream to delete.
	 */
	void WINAPI AsagiriDeleteBinary(HANDLE hAsagiri, LPBYTE lpBin);


#ifdef __cplusplus
}
#endif

#ifndef _UNICODE
#define AsagiriEnumNext				AsagiriEnumNextA
#define AsagiriGetText				AsagiriGetTextA
#define AsagiriGetBitmap			AsagiriGetBitmapA
#define AsagiriGetBinary			AsagiriGetBinaryA
#define AsagiriGetImageFileInBinary	AsagiriGetImageFileInBinaryA
#define AsagiriGetFileName			AsagiriGetFileNameA
#define AsagiriAddPopup				AsagiriAddPopupA
#define AsagiriAddText				AsagiriAddTextA
#define AsagiriAddBitmap			AsagiriAddBitmapA
#define AsagiriAddBinary			AsagiriAddBinaryA
#define AsagiriAddFile				AsagiriAddFileA
#define AsagiriDeleteString			AsagiriDeleteStringA
#else
#define AsagiriEnumNext				AsagiriEnumNextW
#define AsagiriGetText				AsagiriGetTextW
#define AsagiriGetBitmap			AsagiriGetBitmapW
#define AsagiriGetBinary			AsagiriGetBinaryW
#define AsagiriGetImageFileInBinary	AsagiriGetImageFileInBinaryW
#define AsagiriGetFileName			AsagiriGetFileNameW
#define AsagiriAddPopup				AsagiriAddPopupW
#define AsagiriAddText				AsagiriAddTextW
#define AsagiriAddBitmap			AsagiriAddBitmapW
#define AsagiriAddBinary			AsagiriAddBinaryW
#define AsagiriAddFile				AsagiriAddFileW
#define AsagiriDeleteString			AsagiriDeleteStringW
#endif

#endif /*__INCLUDE_ASAGIRI_SDK__*/
